@@ -2,7 +2,8 @@
  * Seahorse
  * 
  * Copyright (C) 2008 Stefan Walter
- * 
+ * Copyright (C) 2011 Collabora Ltd.
+ *
  * This program is free software; you can redistribute it and/or modify 
  * it under the terms of the GNU Lesser General Public License as
  * published by the Free Software Foundation; either version 2.1 of
@@ -28,8 +29,6 @@
 #include "seahorse-windows.h"
 #include "seahorse-keyserver-results.h"
 
-#include "seahorse-gconf.h"
-#include "seahorse-operation.h"
 #include "seahorse-progress.h"
 #include "seahorse-util.h"
 
@@ -37,6 +36,24 @@
 
 #include <glib/gi18n.h>
 
+void           on_keymanager_row_activated              (GtkTreeView* view,
+                                                         GtkTreePath* path,
+                                                         GtkTreeViewColumn* column,
+                                                         SeahorseKeyManager* self);
+
+gboolean       on_keymanager_key_list_button_pressed    (GtkTreeView* view,
+                                                         GdkEventButton* event,
+                                                         SeahorseKeyManager* self);
+
+gboolean       on_keymanager_key_list_popup_menu        (GtkTreeView* view,
+                                                         SeahorseKeyManager* self);
+
+void           on_keymanager_new_button                 (GtkButton* button,
+                                                         SeahorseKeyManager* self);
+
+void           on_keymanager_import_button              (GtkButton* button,
+                                                         SeahorseKeyManager* self);
+
 enum {
 	PROP_0,
 	PROP_SELECTED
@@ -57,6 +74,7 @@ struct _SeahorseKeyManagerPrivate {
 	GQuark track_selected_id;
 	guint track_selected_tab;
 	TabInfo* tabs;
+	GSettings *settings;
 };
 
 enum  {
@@ -357,7 +375,7 @@ initialize_tab (SeahorseKeyManager* self, const char* tabwidget, guint tabid, co
 	gtk_widget_realize (GTK_WIDGET (view));
 
 	/* Add new key store and associate it */
-	self->pv->tabs[tabid].store = seahorse_key_manager_store_new (objects, view);
+	self->pv->tabs[tabid].store = seahorse_key_manager_store_new (objects, view, self->pv->settings);
 }
 
 static gboolean 
@@ -401,35 +419,37 @@ on_filter_changed (GtkEntry* entry, SeahorseKeyManager* self)
 }
 
 static void 
-imported_keys (SeahorseOperation* op, SeahorseKeyManager* self) 
+on_import_complete (GObject *source,
+                    GAsyncResult *result,
+                    gpointer user_data)
 {
-	g_return_if_fail (SEAHORSE_IS_KEY_MANAGER (self));
-	g_return_if_fail (SEAHORSE_IS_OPERATION (op));
-	
-	if (!seahorse_operation_is_successful (op)) {
-		seahorse_operation_display_error (op, _("Couldn't import keys"), 
-		                                  GTK_WIDGET (seahorse_viewer_get_window (SEAHORSE_VIEWER (self))));
-		return;
+	SeahorseKeyManager* self = SEAHORSE_KEY_MANAGER (user_data);
+	GError *error = NULL;
+
+	if (!seahorse_source_import_finish (SEAHORSE_SOURCE (source), result, &error)) {
+		seahorse_util_handle_error (&error, seahorse_viewer_get_window (SEAHORSE_VIEWER (self)),
+		                            "%s", _("Couldn't import keys"));
+	} else {
+		seahorse_viewer_set_status (SEAHORSE_VIEWER (self), _("Imported keys"));
 	}
-	
-	seahorse_viewer_set_status (SEAHORSE_VIEWER (self), _("Imported keys"));
+
+	g_object_unref (self);
 }
 
 static void 
 import_files (SeahorseKeyManager* self, const gchar** uris) 
 {
 	GError *error = NULL;
-	SeahorseMultiOperation* mop;
 	GFileInputStream* input;
-	SeahorseOperation* op;
+	GCancellable *cancellable;
 	const gchar *uri;
 	GString *errmsg;
 	GFile* file;
-	
+
 	g_return_if_fail (SEAHORSE_IS_KEY_MANAGER (self));
-	mop = g_object_new (SEAHORSE_TYPE_MULTI_OPERATION, NULL);
 	errmsg = g_string_new ("");
-	
+	cancellable = g_cancellable_new ();
+
 	for (uri = *uris; uri; uris++, uri = *uris) {
 		GQuark ktype;
 		SeahorseSource* sksrc;
@@ -454,23 +474,20 @@ import_files (SeahorseKeyManager* self, const gchar** uris)
 			g_string_append_printf (errmsg, "%s: %s\n", uri, error->message);
 			g_clear_error (&error);
 			continue;
-		}			
-		
-		op = seahorse_source_import (sksrc, G_INPUT_STREAM (input));
-		seahorse_multi_operation_take (mop, op);
+		}
+
+		seahorse_source_import_async (sksrc, G_INPUT_STREAM (input),
+		                              cancellable, on_import_complete,
+		                              g_object_ref (self));
 	}
-	
-	if (seahorse_operation_is_running (SEAHORSE_OPERATION (mop))) {
-		seahorse_progress_show (SEAHORSE_OPERATION (mop), _("Importing keys"), TRUE);
-		seahorse_operation_watch (SEAHORSE_OPERATION (mop), (SeahorseDoneFunc)imported_keys, self, NULL, NULL);
-	}
-	
+
+	seahorse_progress_show (cancellable, _("Importing keys"), TRUE);
+	g_object_unref (cancellable);
+
 	if (errmsg->len > 0)
 		seahorse_util_show_error (GTK_WIDGET (seahorse_viewer_get_window (SEAHORSE_VIEWER (self))), 
 		                          _("Couldn't import keys"), errmsg->str);
 
-	
-	g_object_unref (mop);
 	g_string_free (errmsg, TRUE);
 }
 
@@ -521,8 +538,8 @@ import_text (SeahorseKeyManager* self, const char* text)
 	GQuark ktype;
 	SeahorseSource* sksrc;
 	GMemoryInputStream* input;
-	SeahorseOperation* op;
-	
+	GCancellable *cancellable;
+
 	g_return_if_fail (SEAHORSE_IS_KEY_MANAGER (self));
 	g_return_if_fail (text != NULL);
 	
@@ -538,14 +555,17 @@ import_text (SeahorseKeyManager* self, const char* text)
 	sksrc = seahorse_context_find_source (seahorse_context_instance (), ktype, SEAHORSE_LOCATION_LOCAL);
 	g_return_if_fail (sksrc != NULL);
 
-	input = seahorse_util_memory_input_string (text, len);
-	op = seahorse_source_import (sksrc, G_INPUT_STREAM (input));
-	
-	seahorse_progress_show (op, _("Importing Keys"), TRUE);
-	seahorse_operation_watch (op, (SeahorseDoneFunc)imported_keys, self, NULL, NULL);
+	input = G_MEMORY_INPUT_STREAM (g_memory_input_stream_new_from_data (g_strndup (text, len),
+	                                                                    len, g_free));
+
+	cancellable = g_cancellable_new ();
+	seahorse_source_import_async (sksrc, G_INPUT_STREAM (input), cancellable,
+	                              on_import_complete, g_object_ref (self));
+
+	seahorse_progress_show (cancellable, _("Importing Keys"), TRUE);
+	g_object_unref (cancellable);
 
 	g_object_unref (input);
-	g_object_unref (op);
 }
 
 static void 
@@ -660,7 +680,7 @@ on_view_type_activate (GtkToggleAction* action, SeahorseKeyManager* self)
 {
 	g_return_if_fail (SEAHORSE_IS_KEY_MANAGER (self));
 	g_return_if_fail (GTK_IS_TOGGLE_ACTION (action));
-	seahorse_gconf_set_boolean (SHOW_TYPE_KEY, gtk_toggle_action_get_active (action));
+	g_settings_set_boolean (self->pv->settings, "show-type", gtk_toggle_action_get_active (action));
 }
 
 
@@ -669,7 +689,7 @@ on_view_expires_activate (GtkToggleAction* action, SeahorseKeyManager* self)
 {
 	g_return_if_fail (SEAHORSE_IS_KEY_MANAGER (self));
 	g_return_if_fail (GTK_IS_TOGGLE_ACTION (action));
-	seahorse_gconf_set_boolean (SHOW_EXPIRES_KEY, gtk_toggle_action_get_active (action));
+	g_settings_set_boolean (self->pv->settings, "show-expiry", gtk_toggle_action_get_active (action));
 }
 
 
@@ -678,7 +698,7 @@ on_view_validity_activate (GtkToggleAction* action, SeahorseKeyManager* self)
 {
 	g_return_if_fail (SEAHORSE_IS_KEY_MANAGER (self));
 	g_return_if_fail (GTK_IS_TOGGLE_ACTION (action));
-	seahorse_gconf_set_boolean (SHOW_VALIDITY_KEY, gtk_toggle_action_get_active (action));
+	g_settings_set_boolean (self->pv->settings, "show-validity", gtk_toggle_action_get_active (action));
 }
 
 static void 
@@ -686,46 +706,32 @@ on_view_trust_activate (GtkToggleAction* action, SeahorseKeyManager* self)
 {
 	g_return_if_fail (SEAHORSE_IS_KEY_MANAGER (self));
 	g_return_if_fail (GTK_IS_TOGGLE_ACTION (action));
-	seahorse_gconf_set_boolean (SHOW_TRUST_KEY, gtk_toggle_action_get_active (action));
-}
-
-static void 
-on_gconf_notify (GConfClient* client, guint cnxn_id, GConfEntry* entry, SeahorseKeyManager* self) 
-{
-	GtkToggleAction* action;
-	const gchar* key;
-	const char* name;
-
-	g_return_if_fail (SEAHORSE_IS_KEY_MANAGER (self));
-	g_return_if_fail (GCONF_IS_CLIENT (client));
-	g_return_if_fail (entry != NULL);
-	
-	key = entry->key;
-	g_return_if_fail (key);
-	
-	if (g_str_equal (key, SHOW_TRUST_KEY)) 
-		name = "view-trust";
-	else if (g_str_equal (key, SHOW_TYPE_KEY)) 
-		name = "view-type";
-	else if (g_str_equal (key, SHOW_EXPIRES_KEY)) 
-		name = "view-expires";
-	else if (g_str_equal (key, SHOW_VALIDITY_KEY)) 
-		name = "view-validity";
-	else
-		return;
-	
-	action = GTK_TOGGLE_ACTION (gtk_action_group_get_action (self->pv->view_actions, name));
-	g_return_if_fail (action != NULL);
-	
-	gtk_toggle_action_set_active (action, gconf_value_get_bool (entry->value));
+	g_settings_set_boolean (self->pv->settings, "show-trust", gtk_toggle_action_get_active (action));
 }
 
 static void
-on_refreshing (SeahorseContext *sctx, SeahorseOperation *operation, SeahorseWidget *swidget)
+on_manager_settings_changed (GSettings *settings, const gchar *key, gpointer user_data)
 {
-	seahorse_progress_status_set_operation (swidget, operation);
-}
+	SeahorseKeyManager *self = SEAHORSE_KEY_MANAGER (user_data);
+	GtkToggleAction* action;
+	const gchar* name;
 
+	if (g_str_equal (key, "show-trust"))
+		name = "view-trust";
+	else if (g_str_equal (key, "show-type"))
+		name = "view-type";
+	else if (g_str_equal (key, "show-expiry"))
+		name = "view-expires";
+	else if (g_str_equal (key, "show-validity"))
+		name = "view-validity";
+	else
+		return;
+
+	action = GTK_TOGGLE_ACTION (gtk_action_group_get_action (self->pv->view_actions, name));
+	g_return_if_fail (action != NULL);
+
+	gtk_toggle_action_set_active (action, g_settings_get_boolean (settings, key));
+}
 
 static const GtkActionEntry GENERAL_ENTRIES[] = {
 	{ "remote-menu", NULL, N_("_Remote") }, 
@@ -845,21 +851,21 @@ seahorse_key_manager_set_selected (SeahorseViewer* base, SeahorseObject* value)
 	g_object_notify (G_OBJECT (self), "selected");
 }
 
-static GObject* 
-seahorse_key_manager_constructor (GType type, guint n_props, GObjectConstructParam *props) 
+static void
+seahorse_key_manager_constructed (GObject *object)
 {
-	SeahorseKeyManager *self = SEAHORSE_KEY_MANAGER (G_OBJECT_CLASS (seahorse_key_manager_parent_class)->constructor(type, n_props, props));
+	SeahorseKeyManager *self = SEAHORSE_KEY_MANAGER (object);
 	GtkActionGroup* actions;
 	GtkToggleAction* action;
 	GtkTargetList* targets;
 	GtkWidget* widget;
 
-	g_return_val_if_fail (self, NULL);	
+	G_OBJECT_CLASS (seahorse_key_manager_parent_class)->constructed (object);
 
 	self->pv->tabs = g_new0 (TabInfo, TAB_NUM_TABS);
 
 	self->pv->notebook = GTK_NOTEBOOK (seahorse_widget_get_widget (SEAHORSE_WIDGET (self), "notebook"));
-	gtk_window_set_title (seahorse_viewer_get_window (SEAHORSE_VIEWER (self)), _("Passwords and Encryption Keys"));
+	gtk_window_set_title (seahorse_viewer_get_window (SEAHORSE_VIEWER (self)), _("Passwords and Keys"));
 	
 	actions = gtk_action_group_new ("general");
 	gtk_action_group_set_translation_domain (actions, GETTEXT_PACKAGE);
@@ -877,18 +883,18 @@ seahorse_key_manager_constructor (GType type, guint n_props, GObjectConstructPar
 	gtk_action_group_set_translation_domain (self->pv->view_actions, GETTEXT_PACKAGE);
 	gtk_action_group_add_toggle_actions (self->pv->view_actions, VIEW_ENTRIES, G_N_ELEMENTS (VIEW_ENTRIES), self);
 	action = GTK_TOGGLE_ACTION (gtk_action_group_get_action (self->pv->view_actions, "view-type"));
-	gtk_toggle_action_set_active (action, seahorse_gconf_get_boolean (SHOW_TYPE_KEY));
+	gtk_toggle_action_set_active (action, g_settings_get_boolean (self->pv->settings, "show-type"));
 	action = GTK_TOGGLE_ACTION (gtk_action_group_get_action (self->pv->view_actions, "view-expires"));
-	gtk_toggle_action_set_active (action, seahorse_gconf_get_boolean (SHOW_EXPIRES_KEY));
+	gtk_toggle_action_set_active (action, g_settings_get_boolean (self->pv->settings, "show-expiry"));
 	action = GTK_TOGGLE_ACTION (gtk_action_group_get_action (self->pv->view_actions, "view-trust"));
-	gtk_toggle_action_set_active (action, seahorse_gconf_get_boolean (SHOW_TRUST_KEY));
+	gtk_toggle_action_set_active (action, g_settings_get_boolean (self->pv->settings, "show-trust"));
 	action = GTK_TOGGLE_ACTION (gtk_action_group_get_action (self->pv->view_actions, "view-validity"));
-	gtk_toggle_action_set_active (action, seahorse_gconf_get_boolean (SHOW_VALIDITY_KEY));
+	gtk_toggle_action_set_active (action, g_settings_get_boolean (self->pv->settings, "show-validity"));
 	seahorse_viewer_include_actions (SEAHORSE_VIEWER (self), self->pv->view_actions);
 	
-	/* Notify us when gconf stuff changes under this key */
-	seahorse_gconf_notify_lazy (LISTING_SCHEMAS, (GConfClientNotifyFunc)on_gconf_notify, self, self);
-	
+	/* Notify us when settings change */
+	g_signal_connect_object (self->pv->settings, "changed", G_CALLBACK (on_manager_settings_changed), self, 0);
+
 	/* close event */
 	g_signal_connect_object (seahorse_widget_get_toplevel (SEAHORSE_WIDGET (self)), 
 	                         "delete-event", G_CALLBACK (on_delete_event), self, 0);
@@ -931,7 +937,7 @@ seahorse_key_manager_constructor (GType type, guint n_props, GObjectConstructPar
 				gtk_toolbar_insert (toolbar, GTK_TOOL_ITEM (sep), -1);
 				
 				/* Insert a filter bar */
-				box = GTK_BOX (gtk_hbox_new (FALSE, 0));
+				box = GTK_BOX (gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0));
 				gtk_box_pack_start (box, GTK_WIDGET (gtk_label_new (_("Filter:"))), FALSE, TRUE, 3);
 				
 				self->pv->filter_entry = GTK_ENTRY (gtk_entry_new ());
@@ -997,17 +1003,13 @@ seahorse_key_manager_constructor (GType type, guint n_props, GObjectConstructPar
 	
 	/* To show first time dialog */
 	g_timeout_add_seconds (1, (GSourceFunc)on_first_timer, self);
-	
-	g_signal_connect (seahorse_context_instance (), "refreshing", G_CALLBACK (on_refreshing), self);
-
-	return G_OBJECT (self);
 }
 
 static void
 seahorse_key_manager_init (SeahorseKeyManager *self)
 {
 	self->pv = G_TYPE_INSTANCE_GET_PRIVATE (self, SEAHORSE_TYPE_KEY_MANAGER, SeahorseKeyManagerPrivate);
-
+	self->pv->settings = g_settings_new ("org.gnome.seahorse.manager");
 }
 
 static void
@@ -1033,6 +1035,8 @@ seahorse_key_manager_finalize (GObject *obj)
 		self->pv->tabs = NULL;
 	}
 
+	g_clear_object (&self->pv->settings);
+
 	G_OBJECT_CLASS (seahorse_key_manager_parent_class)->finalize (obj);
 }
 
@@ -1076,7 +1080,7 @@ seahorse_key_manager_class_init (SeahorseKeyManagerClass *klass)
 	seahorse_key_manager_parent_class = g_type_class_peek_parent (klass);
 	g_type_class_add_private (klass, sizeof (SeahorseKeyManagerPrivate));
 
-	gobject_class->constructor = seahorse_key_manager_constructor;
+	gobject_class->constructed = seahorse_key_manager_constructed;
 	gobject_class->finalize = seahorse_key_manager_finalize;
 	gobject_class->set_property = seahorse_key_manager_set_property;
 	gobject_class->get_property = seahorse_key_manager_get_property;
